/* This file is part of the KDE project
   Copyright (C) 1998, 1999 David Faure <faure@kde.org>
   Copyright (C) 2001 Holger Freyther <freyther@yahoo.com>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#ifndef __konqpopupmenu_h
#define __konqpopupmenu_h

#include <sys/types.h>

#include <qpopupmenu.h>
#include <qmap.h>
#include <kaction.h>

#include <qstringlist.h>

#include <kfileitem.h>
#include <kmimetype.h> // for KDEDesktopMimeType
#include <qregexp.h>
#include <vector>
#include <iterator>
#include <list>
#include <map>


#include <kparts/browserextension.h>

#include "konq_xmlguiclient.h"
#include "konq_dirprot.h"

using namespace std;


typedef QValueList<KDEDesktopMimeType::Service> ServiceList;

class KNewMenu;
class KService;
class KonqPopupMenuPlugin;
class KBookmarkManager;

typedef void (*ACTIONFUN)();
typedef void (*INITIALIZE)(KFileItemList*);
typedef bool (*EXTLIMIT)();

class KonqAddinItem
{
public:
       QRegExp m_expLimit;
       QString m_strSection;
       QString m_strParentSection;
       QString m_strModule;
       QString m_strMenu;
       QString m_strIcon;
       QString m_strExec;
       QString m_strArgs;
       int m_nMultiItem;       // 0: no , 1: yes , 2: only
       int m_nDirMode;
       int m_nRemote;
       int m_nSeperator;      // add a seperator, 0: no , 1: top, 2: bottom, 3: both
       bool m_bGroup;  // Whether is a group
//     bool m_bEmpty;  // if a group, whether temperory and empty
       bool m_bInGroup;        // add in lists if m_bInGroup==false
       bool m_bFullUrl;        // the parameter been sent, whether pathname
       bool m_bNegExp;
       bool m_bRootOnly;
       list<KonqAddinItem*> m_lstSubItems;
       KonqAddinItem(QString &section)
       {
               m_expLimit.setPattern(".+");
               m_strSection=section;
               m_strParentSection="";
               m_strModule="";
               m_strMenu="";
               m_strExec="";
               m_strIcon="";
               m_strArgs="";
               m_nMultiItem=0;
               m_nDirMode=0;
               m_nRemote=0;
               m_bGroup=false;
               m_bInGroup=false;
               m_bFullUrl=false;
               m_bNegExp=false;
               m_bRootOnly=false;
       }
       virtual ~KonqAddinItem(){}
};

class KActionReceiver : public QObject
{
       Q_OBJECT
public:
       KActionReceiver(const char* pModule,QString& strExec,QString &strArg,KFileItemList* pList,bool bFullUrl);
       virtual ~KActionReceiver();
public slots:
       void OnFireMenu();
private:
       ACTIONFUN m_pActFun;
       EXTLIMIT m_pLimit;
       INITIALIZE m_pInit;
       void* m_pHandle;
       QString m_strExe;
       QString m_strArg;
       KFileItemList *m_pList;
       bool m_bFullUrl;
friend class KonqPopupMenu;
};

/**
 * This class implements the popup menu for URLs in konqueror and kdesktop
 * It's usage is very simple : on right click, create the KonqPopupMenu instance
 * with the correct arguments, then exec() to make it appear, then destroy it.
 *
 */
class KonqPopupMenu : public QPopupMenu, public KonqXMLGUIClient
{
  Q_OBJECT
public:

  /**
   * Flags set by the calling application (konqueror/kdesktop), unlike
   * KParts::BrowserExtension::PopupFlags, which are set by the calling part
   */
  typedef uint KonqPopupFlags;
  enum { NoFlags = 0,
         ShowProperties = 1,  // whether to show the "Properties" menu item
         IsLink = 2,          // HTML link. If set, we won't have cut/copy/paste, and we'll say "bookmark this link"
         ShowNewWindow = 4 };
         // WARNING: bitfield. Next item is 8

  /**
   * @deprecated, lacks parentWidget pointer, and
   * uses bool instead of KonqPopupFlags enum,
   * might do strange things with the 'new window' action...
   */
  KonqPopupMenu( KBookmarkManager* manager,
                 const KFileItemList &items,
                 KURL viewURL,
                 KActionCollection & actions,
                 KNewMenu * newMenu,
                 bool showPropertiesAndFileType = true ) KDE_DEPRECATED;

  /**
   * @deprecated, uses bool instead of KonqPopupFlags enum,
   * might do strange things with the 'new window' action...
   */
  KonqPopupMenu( KBookmarkManager* manager,
                 const KFileItemList &items,
                 KURL viewURL,
                 KActionCollection & actions,
                 KNewMenu * newMenu,
		 QWidget * parentWidget,
		 bool showPropertiesAndFileType = true ) KDE_DEPRECATED;

  /**
   * Constructor
   * @param items the list of file items the popupmenu should be shown for
   * @param viewURL the URL shown in the view, to test for RMB click on view background
   * @param actions list of actions the caller wants to see in the menu
   * @param newMenu "New" menu, shared with the File menu, in konqueror
   * @param parentWidget the widget we're showing this popup for. Helps destroying
   * the popup if the widget is destroyed before the popup.
   * @param kpf flags from the KonqPopupFlags enum, set by the calling application
   * @param f flags from the BrowserExtension enum, set by the calling part
   *
   * The actions to pass in include :
   * showmenubar, back, forward, up, cut, copy, paste, pasteto, trash, rename, del
   * (TODO: that list is probably not be up-to-date)
   * The others items are automatically inserted.
   *
   * @since 3.2
   */
  KonqPopupMenu( KBookmarkManager* manager,
		 const KFileItemList &items,
                 const KURL& viewURL,
                 KActionCollection & actions,
                 KNewMenu * newMenu,
                 QWidget * parentWidget,
                 KonqPopupFlags kpf,
                 KParts::BrowserExtension::PopupFlags f /*= KParts::BrowserExtension::DefaultPopupItems*/);

  /**
   * Don't forget to destroy the object
   */
  ~KonqPopupMenu();

  /**
   * Set the title of the URL, when the popupmenu is opened for a single URL.
   * This is used if the user chooses to add a bookmark for this URL.
   */
  void setURLTitle( const QString& urlTitle );

  class ProtocolInfo {
   public:
    ProtocolInfo();
    bool supportsReading()  const;
    bool supportsWriting()  const;
    bool supportsDeleting() const;
    bool supportsMoving()   const;
    bool trashIncluded()    const;
   private:
    friend class KonqPopupMenu;
    bool m_Reading:1;
    bool m_Writing:1;
    bool m_Deleting:1;
    bool m_Moving:1;
    bool m_TrashIncluded:1;
  };
  /**
   * Reimplemented for internal purpose
   */
  virtual KAction *action( const QDomElement &element ) const;


  virtual KActionCollection *actionCollection() const;
  QString mimeType( ) const;
  KURL url( ) const;
  KFileItemList fileItemList() const;
  KURL::List popupURLList( ) const;
  ProtocolInfo protocolInfo() const;

public slots:
  void slotPopupNewDir();
  void slotPopupNewView();
  void slotOpenwith();
  void slotPopupEmptyTrashBin();
  void slotPopupShareDlg();
  void slotPopupOpenWith();
  void slotPopupAddToBookmark();
  void slotRunService();
  void slotPopupMimeType();
  void slotPopupProperties();
  void slotOpenShareFileDialog();
protected:
  KActionCollection &m_actions;
  KActionCollection m_ownActions;

private:
  bool parseAddinScripts();
  bool BuildPlugins();
  bool BuildItem(KonqAddinItem* pItem,QDomElement& menu);
  KonqAddinItem* ReadEntries(KConfig& config, KonqAddinItem* pItems);

  void init (QWidget * parentWidget, KonqPopupFlags kpf, KParts::BrowserExtension::PopupFlags itemFlags);
  void setup(KonqPopupFlags kpf);
  void addPlugins( );
  int  insertServicesSubmenus(const QMap<QString, ServiceList>& list, QDomElement& menu, bool isBuiltin);
  int  insertServices(const ServiceList& list, QDomElement& menu, bool isBuiltin);
  bool KIOSKAuthorizedAction(KConfig& cfg);

  class KonqPopupMenuPrivate;
  KonqPopupMenuPrivate *d;
  KNewMenu *m_pMenuNew;
  KURL m_sViewURL;
  QString m_sMimeType;
  KFileItemList m_lstItems;
  KURL::List m_lstPopupURLs;
  QMap<int,KService::Ptr> m_mapPopup;
  QMap<int,KDEDesktopMimeType::Service> m_mapPopupServices;
  bool m_bHandleEditOperations;
  KXMLGUIFactory *m_factory;
  KXMLGUIBuilder *m_builder;
  QString attrName;
  ProtocolInfo m_info;
  QPtrList<KonqPopupMenuPlugin> m_pluginList;
  KBookmarkManager* m_pManager;
  list<KonqAddinItem*> m_lstTopItems;
  list<KActionReceiver*> m_lstActions;
  QDict<KonqAddinItem> m_dicItems;

  bool bTrashIncluded;
  bool bMyComputerIncluded;
  bool bMyDocumentIncluded;
  bool bNetNeighborIncluded;
  QString myComputerPath;
  QString myDocumentPath;
  QString myNetNBPath;

};

class KonqPopupMenuPlugin : public QObject, public KonqXMLGUIClient {
	Q_OBJECT
public:
  /**
  * Constructor
  * If you want to insert a dynamic item or menu to konqpopupmenu
  * this class is the right choice.
  * Create a KAction and use _popup->addAction(new KAction );
  * If you want to create a submenu use _popup->addGroup( );
  */
  KonqPopupMenuPlugin( KonqPopupMenu *_popup, const char *name ); // this should also be the parent
  virtual ~KonqPopupMenuPlugin ( );
};

#endif

